# Version number following semver standard.
version = '0.28.7'

# Default server address for remote runs, if unspecified.
default_server = 'https://server.siliconcompiler.com'

# This is the list of significant contributors to SiliconCompiler in
# chronological order.
#
# This does not necessarily list everyone who has contributed code,
# especially since many employees of one corporation may be contributing.
# To see the full list of contributors, see the git revision history
authors = [
    'Andreas Olofsson',
    'William Ransohoff',
    'Noah Moroze',
    'Zachary Yedidia',
    'Massimiliano Giacometti',
    'Kimia Talaei',
    'Peter Gadfort',
    'Aulihan Teng',
    'Peter Grossmann',
    'Gabriel Aguirre',
    'Martin Troiber'
]

# CLI entry banner autogenerated using pyfiglet.
# >> pyfiglet.figlet_format("Silicon Compiler")
banner = r'''
 ____  _ _ _                    ____                      _ _
/ ___|(_) (_) ___ ___  _ __    / ___|___  _ __ ___  _ __ (_) | ___ _ __
\___ \| | | |/ __/ _ \| '_ \  | |   / _ \| '_ ` _ \| '_ \| | |/ _ \ '__|
 ___) | | | | (_| (_) | | | | | |__| (_) | | | | | | |_) | | |  __/ |
|____/|_|_|_|\___\___/|_| |_|  \____\___/|_| |_| |_| .__/|_|_|\___|_|
                                                   |_|
'''
